@@ -24,7 +24,7 @@ use utf8;
 our $whatToTrace;
 *whatToTrace = \$Lemonldap::NG::Handler::_CGI::whatToTrace;
 
-our $VERSION = '1.2.1';
+our $VERSION = '1.3.0';
 
 our @ISA = qw(
   Lemonldap::NG::Handler::CGI
@@ -80,25 +80,28 @@ sub new {
     # Multi values separator
     $self->{multiValuesSeparator} ||= '; ';
 
-    # Now we're ready to display sessions. Choose display type
-    foreach my $k ( $self->param() ) {
+    # Attributes to hide
+    $self->{hiddenAttributes} = "_password"
+      unless defined $self->{hiddenAttributes};
 
-        # Case ajax request : execute corresponding sub and quit
-        if ( grep { $_ eq $k } qw(delete session id uidByIp uid letter p) ) {
-            print $self->header( -type => 'text/html;charset=utf-8' );
-            print $self->$k( $self->param($k) );
-            $self->quit();
-        }
-
-        # Case else : store tree type choosen to use it later in tree()
-        elsif ( grep { $_ eq $k } qw(doubleIp fullip fulluid ipclasses) ) {
-            $self->{_tree} = $k;
-            last;
-        }
+    # Now we're ready to display sessions. Choose display type:
+    # case AJAX request
+    if ( my ($k) = grep /^(?:uid(?:ByIp)?|session|delete|letter|id|p)$/,
+        $self->param() )
+    {
+        print $self->header( -type => 'text/html;charset=utf-8' );
+        $self->lmLog( "Ajax request: $k", 'debug' );
+        print $self->$k( $self->param($k) );
+        $self->quit();
     }
 
+    # case else : store tree type choosen to use it later in tree()
+    ( $self->{_tree} ) = grep /^(?:full(?:uid|ip)|ipclasses|doubleIp)$/,
+      $self->param();
+
     # default display : list by uid
     $self->{_tree} ||= 'list';
+    $self->lmLog( "Session display type: $self->{_tree}", 'debug' );
 
     return $self;
 }
@@ -134,17 +137,16 @@ sub list {
     $count = 0;
 
     # Parse all sessions to store first letter
-    Lemonldap::NG::Common::Apache::Session->get_key_from_all_sessions(
+    $res = Lemonldap::NG::Common::Apache::Session->get_key_from_all_sessions(
         $self->{globalStorageOptions},
-        sub {
-            my $entry = shift;
-            next if ( $entry->{_httpSessionType} );
-            $entry->{$whatToTrace} =~ /^(\w)/ or return undef;
-            $byUid->{$1}++;
-            $count++;
-            undef;
-        }
-    );
+        [ '_httpSessionType', $whatToTrace ] );
+    while ( my ( $id, $entry ) = each %$res ) {
+        next if ( $entry->{_httpSessionType} );
+        next unless $entry->{$whatToTrace} =~ /^(\w)/;
+        $byUid->{$1}++;
+        $count++;
+    }
+    $res = '';
 
     # Build tree sorted by first letter
     foreach my $letter ( sort keys %$byUid ) {
@@ -185,18 +187,16 @@ sub doubleIp {
     my ( $byUid, $byIp, $res, $count );
 
     # Parse all sessions
-    Lemonldap::NG::Common::Apache::Session->get_key_from_all_sessions(
+    $res = Lemonldap::NG::Common::Apache::Session->get_key_from_all_sessions(
         $self->{globalStorageOptions},
-        sub {
-            my $entry = shift;
-            my $id    = shift;
-            next if ( $entry->{_httpSessionType} );
-            push @{ $byUid->{ $entry->{$whatToTrace} }
-                  ->{ $entry->{ $self->{ipField} } } },
-              { id => $id, startTime => $entry->{startTime} };
-            undef;
-        }
-    );
+        [ '_httpSessionType', $whatToTrace, $self->{ipField}, 'startTime' ] );
+    while ( my ( $id, $entry ) = each %$res ) {
+        next if ( $entry->{_httpSessionType} );
+        push @{ $byUid->{ $entry->{$whatToTrace} }
+              ->{ $entry->{ $self->{ipField} } } },
+          { id => $id, startTime => $entry->{startTime} };
+    }
+    $res = '';
 
     # Build tree sorted by uid (or other field chosen in whatToTrace parameter)
     foreach my $uid (
@@ -247,25 +247,18 @@ sub fullip {
     my ( $self, $req ) = splice @_;
     my ( $byUid, $res );
 
-    # Build regexp based on IP request
-    my $reip = quotemeta($req);
-    $reip =~ s/\\\*/\.\*/g;
-
-    # Parse all sessions and store only if IP match regexp
-    Lemonldap::NG::Common::Apache::Session->get_key_from_all_sessions(
+    # Parse sessions and store only if IP match regexp
+    $res = Lemonldap::NG::Common::Apache::Session->searchOnExpr(
         $self->{globalStorageOptions},
-        sub {
-            my $entry = shift;
-            my $id    = shift;
-            next if ( $entry->{_httpSessionType} );
-            if ( $entry->{ $self->{ipField} } =~ /$reip/ ) {
-                push @{ $byUid->{ $entry->{ $self->{ipField} } }
-                      ->{ $entry->{$whatToTrace} } },
-                  { id => $id, startTime => $entry->{startTime} };
-            }
-            undef;
-        }
-    );
+        $self->{ipField}, $req, $whatToTrace, 'startTime', $self->{ipField},
+        '_httpSessionType' );
+    while ( my ( $id, $entry ) = each %$res ) {
+        next if ( $entry->{_httpSessionType} );
+        push @{ $byUid->{ $entry->{ $self->{ipField} } }
+              ->{ $entry->{$whatToTrace} } },
+          { id => $id, startTime => $entry->{startTime} };
+    }
+    $res = '';
 
     # Build tree sorted by IP
     foreach my $ip ( sort keys %$byUid ) {
@@ -297,26 +290,19 @@ sub fulluid {
     my ( $self, $req ) = splice @_;
     my ( $byUid, $res );
 
-    # Build regexp based on request
-    my $reuser = quotemeta($req);
-    $reuser =~ s/\\\*/\.\*/g;
-
-    # Parse all sessions to find user that match regexp
-    Lemonldap::NG::Common::Apache::Session->get_key_from_all_sessions(
+    # Parse sessions to find user that match regexp
+    $res = Lemonldap::NG::Common::Apache::Session->searchOnExpr(
         $self->{globalStorageOptions},
-        sub {
-            my $entry = shift;
-            my $id    = shift;
-            next if ( $entry->{_httpSessionType} );
-            if ( $entry->{$whatToTrace} =~ /^$reuser$/ ) {
-                push @{ $byUid->{ $entry->{$whatToTrace} } },
-                  { id => $id, startTime => $entry->{startTime} };
-            }
-            undef;
-        }
-    );
+        $whatToTrace, $req, $whatToTrace, 'startTime', '_httpSessionType' );
+    while ( my ( $id, $entry ) = each %$res ) {
+        next if ( $entry->{_httpSessionType} );
+        push @{ $byUid->{ $entry->{$whatToTrace} } },
+          { id => $id, startTime => $entry->{startTime} };
+    }
+    $res = '';
 
     # Build tree sorted by uid
+    $res .= "<li id=\"re\"><span>$req</span><ul>";
     foreach my $uid ( sort keys %$byUid ) {
         $res .= $self->ajaxNode(
             $uid,
@@ -331,6 +317,7 @@ sub fulluid {
             "uid=$uid"
         );
     }
+    $res .= "</ul></li>";
 }
 
 ## @method protected string ipclasses()
@@ -461,8 +448,8 @@ sub session {
             $value = $newvalue;
         }
 
-        # Hide password
-        $value = '******' if ( $_ =~ /^_password$/ );
+        # Hide attributes
+        $value = '****' if ( $self->{hiddenAttributes} =~ /\b$_\b/ );
 
         # Manage timestamp
         if ( $_ =~ /^(_utime|_lastAuthnUTime)$/ ) {
@@ -488,8 +475,9 @@ sub session {
         'saml'         => [
             qw(_idp _idpConfKey _samlToken _lassoSessionDump _lassoIdentityDump)
         ],
-        'groups' => [qw(groups)],
-        'ldap'   => [qw(dn)],
+        'groups'    => [qw(groups)],
+        'ldap'      => [qw(dn)],
+        'BrowserID' => [qw(_browserIdAnswer _browserIdAnswerRaw)],
     };
 
     # Display categories
@@ -567,7 +555,7 @@ sub session {
         foreach ( keys %session ) {
             next if $_ !~ /^notification_(.+)/;
             $res .=
-                '<li><strong>' 
+                '<li><strong>'
               . $1
               . '</strong>: '
               . $session{$_} . " ("
@@ -665,19 +653,18 @@ sub session {
 sub uidByIp {
     my ( $self, $ip ) = splice @_;
     my ( $byUser, $res );
-    Lemonldap::NG::Common::Apache::Session->get_key_from_all_sessions(
+    $res = Lemonldap::NG::Common::Apache::Session->searchOn(
         $self->{globalStorageOptions},
-        sub {
-            my $entry = shift;
-            my $id    = shift;
-            next if ( $entry->{_httpSessionType} );
-            if ( $entry->{ $self->{ipField} } eq $ip ) {
-                push @{ $byUser->{ $entry->{$whatToTrace} } },
-                  { id => $id, startTime => $entry->{startTime} };
-            }
-            undef;
+        $self->{ipField}, $ip, '_httpSessionType', $whatToTrace,
+        $self->{ipField}, 'startTime' );
+    while ( my ( $id, $entry ) = each(%$res) ) {
+        next if ( $entry->{_httpSessionType} );
+        if ( $entry->{ $self->{ipField} } eq $ip ) {
+            push @{ $byUser->{ $entry->{$whatToTrace} } },
+              { id => $id, startTime => $entry->{startTime} };
         }
-    );
+    }
+    $res = '';
     foreach my $user ( sort keys %$byUser ) {
         $res .= "<li id=\"ip$user\"><span>$user</span><ul>";
         foreach my $session ( sort { $a->{startTime} <=> $b->{startTime} }
@@ -699,19 +686,18 @@ sub uidByIp {
 sub uid {
     my ( $self, $uid ) = splice @_;
     my ( $byIp, $res );
-    Lemonldap::NG::Common::Apache::Session->get_key_from_all_sessions(
+    $res = Lemonldap::NG::Common::Apache::Session->searchOn(
         $self->{globalStorageOptions},
-        sub {
-            my $entry = shift;
-            my $id    = shift;
-            next if ( $entry->{_httpSessionType} );
-            if ( $entry->{$whatToTrace} eq $uid ) {
-                push @{ $byIp->{ $entry->{ $self->{ipField} } } },
-                  { id => $id, startTime => $entry->{startTime} };
-            }
-            undef;
+        $whatToTrace, $uid, '_httpSessionType', $whatToTrace,
+        $self->{ipField}, 'startTime' );
+    while ( my ( $id, $entry ) = each(%$res) ) {
+        next if ( $entry->{_httpSessionType} );
+        if ( $entry->{$whatToTrace} eq $uid ) {
+            push @{ $byIp->{ $entry->{ $self->{ipField} } } },
+              { id => $id, startTime => $entry->{startTime} };
         }
-    );
+    }
+    $res = '';
     foreach my $ip ( sort keys %$byIp ) {
         $res .= "<li class=\"open\" id=\"uid$ip\"><span>$ip</span><ul>";
         foreach my $session ( sort { $a->{startTime} <=> $b->{startTime} }
@@ -735,15 +721,15 @@ sub letter {
     my $self   = shift;
     my $letter = $self->param('letter');
     my ( $byUid, $res );
-    Lemonldap::NG::Common::Apache::Session->get_key_from_all_sessions(
+
+    $res = Lemonldap::NG::Common::Apache::Session->searchOnExpr(
         $self->{globalStorageOptions},
-        sub {
-            my $entry = shift;
-            next if ( $entry->{_httpSessionType} );
-            $entry->{$whatToTrace} =~ /^$letter/ or return undef;
-            $byUid->{ $entry->{$whatToTrace} }++;
-        },
-    );
+        $whatToTrace, "${letter}*", '_httpSessionType', $whatToTrace );
+    while ( my ( $id, $entry ) = each %$res ) {
+        next if ( $entry->{_httpSessionType} );
+        $byUid->{ $entry->{$whatToTrace} }++;
+    }
+    $res = '';
     foreach my $uid ( sort keys %$byUid ) {
         $res .= $self->ajaxNode(
             $uid,
@@ -782,19 +768,17 @@ sub _ipclasses {
     my $partial = $p ? "$p." : '';
     my $repartial = quotemeta($partial);
     my ( $byIp, $count, $res );
-    Lemonldap::NG::Common::Apache::Session->get_key_from_all_sessions(
-        $self->{globalStorageOptions},
-        sub {
-            my $entry = shift;
-            next if ( $entry->{_httpSessionType} );
-            $entry->{ $self->{ipField} } =~ /^$repartial(\d+)/
-              or return undef;
-            $byIp->{$1}++;
-            $count++;
-            undef;
-        }
-    );
 
+    $res = Lemonldap::NG::Common::Apache::Session->searchOnExpr(
+        $self->{globalStorageOptions},
+        $self->{ipField}, "${partial}*", '_httpSessionType', $self->{ipField} );
+    while ( my ( $id, $entry ) = each %$res ) {
+        next if ( $entry->{_httpSessionType} );
+        $entry->{ $self->{ipField} } =~ /^$repartial(\d+)/ or next;
+        $byIp->{$1}++;
+        $count++;
+    }
+    $res = '';
     foreach my $ip ( sort { $a <=> $b } keys %$byIp ) {
         $res .= $self->ajaxNode(
             "$partial$ip",
@@ -858,6 +842,8 @@ sub _stToStr {
 1;
 __END__
 
+=encoding utf8
+
 =head1 NAME
 
 Lemonldap::NG::Manager::Sessions - Perl extension to manage Lemonldap::NG
@@ -872,7 +858,7 @@ sessions
   our $cgi ||= Lemonldap::NG::Manager::Sessions->new({
         localStorage        => "Cache::FileCache",
         localStorageOptions => {
-            'namespace'          => 'lemonldap-ng',
+            'namespace'          => 'lemonldap-ng-config',
             'default_expires_in' => 600,
             'directory_umask'    => '007',
             'cache_root'         => '/tmp',
@@ -913,7 +899,7 @@ L<Lemonldap::NG::Handler::CGI>, L<Lemonldap::NG::Manager>
 
 =item Clement Oudot, E<lt>clem.oudot@gmail.comE<gt>
 
-=item Franois-Xavier Deltombe, E<lt>fxdeltombe@gmail.com.E<gt>
+=item François-Xavier Deltombe, E<lt>fxdeltombe@gmail.com.E<gt>
 
 =item Xavier Guimard, E<lt>x.guimard@free.frE<gt>
 
@@ -937,7 +923,7 @@ L<http://forge.objectweb.org/project/showfiles.php?group_id=274>
 
 =item Copyright (C) 2008, 2009, 2010, 2013 by Xavier Guimard, E<lt>x.guimard@free.frE<gt>
 
-=item Copyright (C) 2012 by Franois-Xavier Deltombe, E<lt>fxdeltombe@gmail.com.E<gt>
+=item Copyright (C) 2012 by François-Xavier Deltombe, E<lt>fxdeltombe@gmail.com.E<gt>
 
 =item Copyright (C) 2009, 2010, 2011, 2012 by Clement Oudot, E<lt>clem.oudot@gmail.comE<gt>
 
